-- This function sends a data query to the target or broadcasts if not target is provided
local function replace_tome_data_query(target, bypassthrottle)
    -- Abort if no target is provided
    if not target then
        return
    end
    -- Check that the query should not be throttled
    if not bypassthrottle and Tome_Throttle[string.upper(target)] and Tome_Throttle[string.upper(target)] > os.time() then
        return
    end

    -- Store the target name and message type
    Tome.Data.Error.Target = target
    Tome.Data.Error.Type = "Query"

    -- Send query to a single target
    Command.Message.Send(target, "Tome_Query", "", Tome.Data.SendCallback)

    -- Call the compatibility modules' query function
    for _, item in pairs(Tome.Compat) do
        item.Query(target)
    end

    -- Set the throttle time
    -- (after all the queries have been sennt, or not-sent)
    Tome_Throttle[string.upper(target)] = os.time() + Tome_Config.Throttle
end

-- This function implements a query for RP data from Merisioux
local function replace_tome_compat_merisioux_query(target, broadcast)
    -- Abort if no target is provided
    if not target then
        return
    end

    -- Check if this is a broadcast
    if not broadcast then
	-- no throttle check here, that's done in the caller.

        -- Store the target name and message type
        Tome.Data.Error.Target = target
        Tome.Data.Error.Type = "Query"

        -- Send query to a single target
        Command.Message.Send(target, "rar_query", "merisioux_data", Tome.Compat.Merisioux.SendCallback)
    else
        -- Broadcast query to anyone in /say range
        Command.Message.Broadcast(target, nil, "rar_query", "merisioux_data")
    end
end

-- This function updates the tooltip with a specified unit
local function replace_tome_tooltip_update(data)
    -- Check if data was manually supplied
    if not data then
        -- Convert the target name to uppercase
	data = Tome.Data.Get(Tome.Tooltip.Target.name)

        if not data then
            Tome.Tooltip.Frame:SetVisible(false)
            return
        end
    end

    -- Set the players title
    Tome.Tooltip.Title:SetText(data.Title)

    -- Make a temporary variable with the player name
    local name = data.Name

    -- Prepend the prefix if the player has one
    if data.Prefix and data.Prefix ~= "" then
        name = string.format("%s %s", data.Prefix, name)
    end

    -- Append the suffix if the player has one
    if data.Suffix and data.Suffix ~= "" then
        name = string.format("%s %s", name, data.Suffix)
    end

    -- Set the players name
    Tome.Tooltip.Name:SetText(name)

    -- Color the name of certain developers and contributors.
    if Tome.Tooltip.Target.name == "Maiiel" then
        Tome.Tooltip.Name:SetFontColor(1.0, 0.4, 0.0, 1.0)
    elseif Tome.Tooltip.Target.name == "Lyrai" then
        Tome.Tooltip.Name:SetFontColor(1.0, 0.0, 0.8, 1.0)
    elseif Tome.Tooltip.Target.name == "Lahni" then
        Tome.Tooltip.Name:SetFontColor(0.6, 0.0, 0.8, 1.0)
    elseif Tome.Tooltip.Target.name == "Asalah" then
        Tome.Tooltip.Name:SetFontColor(0.9, 1.0, 0.1, 1.0)
    else
        Tome.Tooltip.Name:SetFontColor(1.0, 1.0, 1.0, 1.0)
    end

    -- Make a temporary variable to store our flag text
    local flagtext = ""

    -- Loop the flags
    for _, flag in pairs(Tome.Data.Flags) do
        -- Check if this is the players flag and set the text if it is
        if flag.id == data.Flag then
            flagtext = flag.text
        end
    end

    -- Check if the player is in tutor mode
    if data.Tutor then
        flagtext = string.format("%s (Tutor)", flagtext)
    end

    -- Set the players flag
    Tome.Tooltip.Flag:SetText(flagtext)

    -- Check if the player is in character
    if data.InCharacter then
        -- Set the players in character status and update the text color
        Tome.Tooltip.InCharacter:SetText("In Character")
        Tome.Tooltip.InCharacter:SetFontColor(0.0, 1.0, 0.0, 1.0)
    else
        -- Set the players in character status and update the text color
        Tome.Tooltip.InCharacter:SetText("Out of Character")
        Tome.Tooltip.InCharacter:SetFontColor(0.78, 0.08, 0.08, 1.0)
    end

    -- Create a temporary variable to store the currently data
    local currently = data.Currently

    -- Enclose the currently data if the player has any
    if data.Currently and data.Currently ~= "" then
        currently = string.format("'Currently: %s'", data.Currently)
    else
        currently = ""
    end

    -- Set the player's currently text
    Tome.Tooltip.Currently:SetText(currently)

    -- Display a message if the data is old
    if data.Expired then
        Tome.Tooltip.Cache:SetText("Expired")
    else
        Tome.Tooltip.Cache:SetText(" ")
    end

    -- Set the origin addon text
    if data.Origin then
        if data.Origin == "Tome" then
            local version = Tome.GetVersion()
            Tome.Tooltip.Origin:SetText(string.format(
                "%s %s.%s.%s%s",
                data.Origin,
                version.Major,
                version.Minor,
                version.Hotfix,
                version.Beta and "-beta" or ""
            ))
        else
            Tome.Tooltip.Origin:SetText(data.Origin)
        end
    end

    -- Update the tooltip width
    Tome.Tooltip.UpdateWidth()

    -- Update the tooltip height
    Tome.Tooltip.UpdateHeight()

    -- Show the frame
    Tome.Tooltip.Frame:SetVisible(true)
end

if Tome then
    print("Attempting to patch Tome to fix caching bug.")
    if Tome.Data then
        Tome.Data.Query = replace_tome_data_query
    end
    if Tome.Compat then
        if Tome.Compat.Merisioux then
	    Tome.Compat.Merisioux.Query = replace_tome_compat_merisioux_query
	end
    end
    if Tome.Tooltip then
        Tome.Tooltip.Update = replace_tome_tooltip_update
    end
end
